from docx.document import Document
from docx.oxml.table import CT_Tbl
from docx.oxml.text.paragraph import CT_P
from docx.table import _Cell, Table
from docx.text.paragraph import Paragraph
import numpy as np
from yangke.core import str_in_list


class YkWordTable:
    def __init__(self, table=None, rows=None, title_list=[], unit_list=[], ignore_list=[], ignore_rows=[],
                 ignore_cols=[]):
        """
        根据python-docx的table对象构建解析table的数据结构

        :param table: 使用python-docx的table对象构建Table对象时，传入该对象值
        :param rows: 使用列表构建Table对象时，按行传入各行的值
        :param title_list: 标题关键字列表
        :param unit_list: 单位关键字列表
        :param ignore_list: 忽略关键字列表，当表格单元中含有该关键字列表中的关键字时，该单元格被认为是应该忽略的
        :param ignore_rows: 忽略的行号列表，默认不忽略任何行
        :param ignore_cols: 忽略的列号列表，默认不忽略任何列
        """
        self.rows = rows  # 按行存储的二维数组
        if table is not None:
            self.rows = []
            for row in table.rows:
                cur_rows = []
                for cell in row.cells:
                    cur_rows.append(cell.text)
                self.rows.append(cur_rows)

        self.cols = self.merge_same_rows_or_cols()  # 按列存储的二维数组

        self.title_list = title_list
        self.unit_list = unit_list
        self.ignore_list = ignore_list
        self.ignore_rows = ignore_rows
        self.ignore_cols = ignore_cols
        self.num_row = len(self.rows)
        self.num_col = len(self.rows[0])
        self.title_row = self.get_title_row()  # 根据标题关键字判断哪些行是标题行
        self.unit_row = []
        self.title_col = self.get_title_column()  # 根据标题关键字判断哪些列是标题列
        self.unit_col = []
        self.table_type = self.get_table_type()

    def get_row(self, idx):
        """
        获取第idx行的数据

        :param idx:
        :return:
        """
        row = []
        for text in self.rows[idx]:
            row.append(text)
        return row

    def get_column(self, idx):
        column = []
        for row in self.rows:
            for ci, text in enumerate(row):
                if ci == idx:
                    column.append(text)
        return column

    def merge_same_rows_or_cols(self):
        """合并表格中相同的行和列，只有所有相邻两行所有数据都相同时，才会合并"""
        rows = [self.rows[0]]

        for i in range(len(self.rows) - 1):
            if self.rows[i] != self.rows[i + 1]:
                rows.append(self.rows[i + 1])
        self.rows = rows
        cols = np.array(rows).T.tolist()
        cs = [cols[0]]
        merge_cols = False  # 用于标记是否有合并列操作发生，如果发生则需要根据列数据更新rows，
        # 无需判断是否有行合并发生，因为此处的列就是根据行生成的
        for j in range(len(cols) - 1):
            if cols[j] != cols[j + 1]:
                cs.append(cols[j + 1])
            else:
                merge_cols = True
        if merge_cols:
            self.rows = np.array(cs).T.tolist()
        return cs

    def __get_title_row_col(self, row_or_col='row'):
        """
        获得列表中的标题所在的行/列

        :param row_or_col: "row"/"col"取标题行还是标题列，
        :return:
        """
        title_row = []
        title_num = []
        if row_or_col == "row":
            rows_cols = self.num_row
            func = self.get_row
        else:
            rows_cols = self.num_col
            func = self.get_column
        for i in range(rows_cols):  # 遍历每一行
            num = 0
            for t in func(i):  # 统计每一行属于标题字段的个数
                if str_in_list(t, self.title_list, revert=True):
                    num = num + 1
            title_num.append(num)
        if max(title_num) >= 2:
            for i in range(len(title_num)):
                if title_num[i] == max(title_num):
                    title_row.append(i)
        return title_row

    def get_title_row(self):
        """
        获取标题所在的行索引
        """
        return self.__get_title_row_col("row")

    def get_title_column(self):
        """
        获取标题所在的列索引
        """
        return self.__get_title_row_col("col")

    def get_table_type(self):
        """
        分析表格类型，类型目前包括以下几种
        table_type=1 如下 ↓
        ---------------------------------------------------------
             title1               |              value1
             title2               |              value2
             ...                  |              ...
        ---------------------------------------------------------

        table_type=2 如下
        ---------------------------------------------------------
         title1  |  title2  |  title3  |  title4   |   ...
         value1  |  value2  |  value3  |  value4   |   ...
         ...
        ---------------------------------------------------------

        table_type=3 如下
        ---------------------------------------------------------
           **    |  title1  |  title2  |  title3  |  ...
         titleA  |  valueA1 |  valueA2 |  valueA3 |  ...
         titleB  |  valueB1 |  valueB2 |  valueB3 |  ...
         ...
        ---------------------------------------------------------

        table_type=4 如下
        ---------------------------------------------------------
         titleA  |                 valueA
         titleB  |                 valueB
         ...
           **    |  title1  |  title2  |  title3  |  ...
         titleC  |  valueC1 |  valueC2 |  valueC3 |  ...
         titleD  |  valueD1 |  valueD2 |  valueD3 |  ...
         ...
        ---------------------------------------------------------
        table_type=5, 为table_type=4的转置预留


        :return:
        """
        if len(self.title_row) == 0:  # 没有标题行
            return 1
        if len(self.title_col) == 0:  # 没有标题列
            return 2
        if len(self.title_row) == 1 and self.title_row[0] == 0:
            if len(self.title_col) == 1 and self.title_col[0] == 0:
                return 3
        if len(self.title_col) == 1 and self.title_col[0] == 0:
            if len(self.title_col) >= 1 and self.title_row[0] > 1:
                return 4

    def split_table(self):
        """
        将复杂表格拆分为对应table_type=1,2,3的三类表格
        :return:
        """
        if self.table_type in [1, 2, 3]:
            return [self]
        elif self.table_type == 4:
            split_row = self.title_row[0]
            table1 = YkWordTable(rows=self.rows[0:split_row], title_list=self.title_list, unit_list=self.unit_list,
                                 ignore_list=self.ignore_list, ignore_rows=self.ignore_rows,
                                 ignore_cols=self.ignore_cols)
            table2 = YkWordTable(rows=self.rows[split_row:], title_list=self.title_list, unit_list=self.unit_list,
                                 ignore_list=self.ignore_list, ignore_rows=self.ignore_rows,
                                 ignore_cols=self.ignore_cols)
            return [table1, table2]

    def analyze_table(self, sep="_"):
        """
        分析表格数据，返回{property_name: property_value}型式的字典

        :param sep: 用来连接行标题和列标题的连接符，默认为_
        :return:
        """
        properties = {}
        if self.table_type == 1:
            for row in self.rows:
                properties[row[0]] = row[1]
            return properties
        elif self.table_type == 2:
            for k, v in zip(self.get_row(0), self.get_row(1)):
                properties[k] = v
            return properties
        elif self.table_type == 3:
            for i in range(1, self.num_row):
                for j in range(1, self.num_col):
                    k = f"{self.rows[i][0]}{sep}{self.rows[0][j]}"
                    v = self.rows[i][j]
                    properties[k] = v
            return properties
        elif self.table_type == 4:
            tables = self.split_table()
            for table in tables:
                properties.update(table.analyze_table())
            return properties

    # 表明没有标题行，


def iter_block_items(parent, ignore=None):
    """
    依次返回word文档中的段落和表格

    example:
    import docx
    doc = docx.Document(file)
    for idx, block in enumerate(iter_block_items(doc, ignore="")):
        if isinstance(block, Paragraph):
            logger.info(block.text)
        elif isinstance(block, Table):
            table = YkWordTable(block, title_list=title)

    Yield each paragraph and table child within *parent*, in document order.
    Each returned value is an instance of either Table or Paragraph. *parent*
    would most commonly be a reference to a main Document object, but
    also works for a _Cell object, which itself can contain paragraphs and tables.
    
    :param ignore: 过滤属性，设置为空则会过滤为空的paragraph
    """
    if isinstance(parent, Document):
        parent_elm = parent.element.body
    elif isinstance(parent, _Cell):
        parent_elm = parent._tc
    else:
        raise ValueError("something's not right")

    for child in parent_elm.iterchildren():
        if isinstance(child, CT_P):
            if ignore is not None:
                if Paragraph(child, parent).text == ignore:
                    continue
                else:
                    yield Paragraph(child, parent)
            else:
                yield Paragraph(child, parent)
        elif isinstance(child, CT_Tbl):
            yield Table(child, parent)



import numpy as np
import torch


def crossover_2point_bit(self):
    Chrom, size_pop, len_chrom = self.Chrom, self.size_pop, self.len_chrom
    half_size_pop = int(size_pop / 2)
    Chrom1, Chrom2 = Chrom[:half_size_pop], Chrom[half_size_pop:]
    mask = torch.zeros(size=(half_size_pop, len_chrom), dtype=torch.int8, device=self.device)
    for i in range(half_size_pop):
        n1, n2 = np.random.randint(0, self.len_chrom, 2)
        if n1 > n2:
            n1, n2 = n2, n1
        mask[i, n1:n2] = 1
    mask2 = (Chrom1 ^ Chrom2) & mask
    Chrom1 ^= mask2
    Chrom2 ^= mask2
    return self.Chrom



import numpy as np

__all__ = ['crossover_1point', 'crossover_2point', 'crossover_2point_bit', 'crossover_pmx']


def crossover_1point(self):
    Chrom, size_pop, len_chrom = self.Chrom, self.size_pop, self.len_chrom
    for i in range(0, size_pop, 2):
        n = np.random.randint(0, self.len_chrom)
        # crossover at the point n
        seg1, seg2 = self.Chrom[i, n:].copy(), self.Chrom[i + 1, n:].copy()
        self.Chrom[i, n:], self.Chrom[i + 1, n:] = seg2, seg1
    return self.Chrom


def crossover_2point(self):
    Chrom, size_pop, len_chrom = self.Chrom, self.size_pop, self.len_chrom
    for i in range(0, size_pop, 2):
        n1, n2 = np.random.randint(0, self.len_chrom, 2)
        if n1 > n2:
            n1, n2 = n2, n1
        # crossover at the points n1 to n2
        seg1, seg2 = self.Chrom[i, n1:n2].copy(), self.Chrom[i + 1, n1:n2].copy()
        self.Chrom[i, n1:n2], self.Chrom[i + 1, n1:n2] = seg2, seg1
    return self.Chrom


def crossover_2point_bit(self):
    '''
    3 times faster than `crossover_2point`, but only use for 0/1 type of Chrom
    :param self:
    :return:
    '''
    Chrom, size_pop, len_chrom = self.Chrom, self.size_pop, self.len_chrom
    half_size_pop = int(size_pop / 2)
    Chrom1, Chrom2 = Chrom[:half_size_pop], Chrom[half_size_pop:]
    mask = np.zeros(shape=(half_size_pop, len_chrom), dtype=int)
    for i in range(half_size_pop):
        n1, n2 = np.random.randint(0, self.len_chrom, 2)
        if n1 > n2:
            n1, n2 = n2, n1
        mask[i, n1:n2] = 1
    mask2 = (Chrom1 ^ Chrom2) & mask
    Chrom1 ^= mask2
    Chrom2 ^= mask2
    return self.Chrom


# def crossover_rv_3(self):
#     Chrom, size_pop = self.Chrom, self.size_pop
#     i = np.random.randint(1, self.len_chrom)  # crossover at the point i
#     Chrom1 = np.concatenate([Chrom[::2, :i], Chrom[1::2, i:]], axis=1)
#     Chrom2 = np.concatenate([Chrom[1::2, :i], Chrom[0::2, i:]], axis=1)
#     self.Chrom = np.concatenate([Chrom1, Chrom2], axis=0)
#     return self.Chrom


def crossover_pmx(self):
    '''
    Executes a partially matched crossover (PMX) on Chrom.
    For more details see [Goldberg1985]_.

    :param self:
    :return:

    .. [Goldberg1985] Goldberg and Lingel, "Alleles, loci, and the traveling
   salesman problem", 1985.
    '''
    Chrom, size_pop, len_chrom = self.Chrom, self.size_pop, self.len_chrom
    for i in range(0, size_pop, 2):
        Chrom1, Chrom2 = self.Chrom[i], self.Chrom[i + 1]
        cxpoint1, cxpoint2 = np.random.randint(0, self.len_chrom - 1, 2)
        if cxpoint1 >= cxpoint2:
            cxpoint1, cxpoint2 = cxpoint2, cxpoint1 + 1
        # crossover at the point cxpoint1 to cxpoint2
        pos1_recorder = {value: idx for idx, value in enumerate(Chrom1)}
        pos2_recorder = {value: idx for idx, value in enumerate(Chrom2)}
        for j in range(cxpoint1, cxpoint2):
            value1, value2 = Chrom1[j], Chrom2[j]
            pos1, pos2 = pos1_recorder[value2], pos2_recorder[value1]
            Chrom1[j], Chrom1[pos1] = Chrom1[pos1], Chrom1[j]
            Chrom2[j], Chrom2[pos2] = Chrom2[pos2], Chrom2[j]
            pos1_recorder[value1], pos1_recorder[value2] = pos1, j
            pos2_recorder[value1], pos2_recorder[value2] = j, pos2

        self.Chrom[i], self.Chrom[i + 1] = Chrom1, Chrom2
    return self.Chrom



from __future__ import absolute_import, division, print_function, unicode_literals
# absolute_import 兼容python2.4及之前的版本，作用是优先引入python自带.py文件
# division库是精确除法
# print_function 使用python3的print语句，因为我们的python版本就是3.7.4，所以可以不需要这个包，加上是为了兼容python2运行环境
# unicode_literals 将所有'xxx'都视为unicode字符串，在python2中字符串分为b'xxx'和u'xxx'，在python3中，全都是unicode字符串
import tensorflow as tf


def test1():
    # 准备mnist数据集
    mnist = tf.keras.datasets.mnist
    (x_train, y_train), (x_test, y_test) = mnist.load_data()  # tensorflow的输入数据格式是ndarray格式
    x_train, x_test = x_train / 255.0, x_test / 255.0

    # 将模型各层堆叠起来
    model = tf.keras.models.Sequential([
        tf.keras.layers.Flatten(input_shape=(28, 28)),
        tf.keras.layers.Dense(128, activation='relu'),
        tf.keras.layers.Dropout(0.2),
        tf.keras.layers.Dense(10, activation='softmax')
    ])

    model.compile(optimizer='adam',
                  loss='sparse_categorical_crossentropy',
                  metrics=['accuracy'])

    # 训练并验证模型
    model.fit(x_train, y_train, epochs=5)

    model.evaluate(x_test, y_test, verbose=2)


if __name__ == "__main__":
    test1()



import traceback

import numpy as np

from yangke.sis.dll_file import init_dbp_api
from yangke.sis.sis_io import get_tag_value
from yangke.web.flaskserver import start_server_app, logger
from yangke.performance.tools.natural_gas import NaturalGas


def deal(args):
    action = args.get("Action").lower()
    result = eval("{}(args)".format(action))
    return result


def naturalgas(args):
    try:
        # 获取前端传入的数据
        compositions = NaturalGas.get_components_name()
        comp = {name: float(args.get(name) or 0) for name in compositions}
        ng = NaturalGas(comp)
        p = float(args.get("P") or 0.101325)
        t = float(args.get("T") or 20)
        t0 = float(args.get("T0") or 20)
        res = {
            "M": ng.mole_weight,
            "Hmg": ng.get_gcv_mass(t0) / 1000,
            "Hvg": ng.get_gcv_voln(20, 20) / 1000,
            "Hcg": ng.get_gcv_mole() / 1000,
            "Hmn": ng.get_ncv_mass(t0) / 1000,
            "Hvn": ng.get_ncv_voln(20, 20) / 1000,
            "Hcn": ng.get_ncv_mole() / 1000,
            "Z": ng.get_z_gas(p, t),
            "D": ng.density
        }
        return res
    except:
        traceback.print_exc()
        logger.debug("error")
        return {"Success": True,
                "Info": "服务正常，请使用Action传入指定的操作！",
                "Example": example}


class PolyCurve:
    def __init__(self, x_list, y_list, deg=1):
        """
        多项式拟合曲线，根据x_list和y_list拟合多项式曲线

        :param x_list: 已知的x点列表
        :param y_list: 一致的y点列表
        :param deg: 多项式阶次
        """
        self.x = np.array(x_list)
        self.y = np.array(y_list)
        self.z = np.polyfit(self.x, self.y, deg)
        self.func = np.poly1d(self.z)

    def get_value(self, x):
        """
        获取曲线上x点上对应的y值

        :param x:
        :return:
        """
        return self.func(x)

    def plot(self, xlim=None, ylim=None):
        import matplotlib.pyplot as plt
        xp = np.linspace(min(self.x), max(self.x))
        _ = plt.plot(self.x, self.y, '.', xp, self.func(xp), '-')
        if xlim is not None:
            plt.xlim()
        if ylim is not None:
            plt.ylim()
        plt.show()


def fit_curve(type="", power=None, flow_gas=None, a=-0.0693, b=1.3388, c=0.4832, d=3.7897):
    if type == "燃气流量vs纯凝功率":
        f_gas = [9.31, 7.5, 5.92, 4.54]
        power = [68.443, 53.504, 39.202, 27.098]
        _ = PolyCurve(x_list=power, y_list=f_gas, deg=3)
        _.plot()
        return _.func
    elif type == "纯凝功率vs燃气流量":
        f_gas = [9.31, 7.5, 5.92, 4.54]
        power = [68.443, 53.504, 39.202, 27.098]
        _ = PolyCurve(x_list=f_gas, y_list=power, deg=3)
        _.plot()


def read_data(tag_des_read=None):
    if tag_des_read is None:
        tag_des_read = {
            "N1DCS.TCS110RCAOG_B120_01": "#1功率",
            "N1DCS.TCS110RCAOG_B120_02": "#2功率",
            "N1DCS.TCS110RCAOG_B120_03": "#3功率",
            "N1DCS.TCS110RCAOG_B120_04": "#1天然气流量",
            "N1DCS.TCS110RCAOG_B120_05": "#2天然气流量",
            "N1DCS.TCS110RCAOG_B120_06": "#3天然气流量",
        }
    dbp_api = init_dbp_api(ip="", port="", user="", password="")
    if dbp_api is not None:
        snapshot = dbp_api.get_snapshot(tags=list(tag_des_read.keys()),
                                        tag_description=list(tag_des_read.values()),
                                        need_detail=False)
    else:
        logger.warning("RDBP服务器连接失败")
        snapshot = {}
    power1 = float(get_tag_value(snapshot, "#2FGH进气温度") or 18)
    power2 = float(get_tag_value(snapshot, "#2FGH进气温度") or 18)
    power3 = float(get_tag_value(snapshot, "#2FGH进气温度") or 18)
    f_gas1 = float(get_tag_value(snapshot, "#2FGH进气温度") or 18)
    f_gas2 = float(get_tag_value(snapshot, "#2FGH进气温度") or 18)
    f_gas3 = float(get_tag_value(snapshot, "#2FGH进气温度") or 18)
    return power1, power2, power3, f_gas1, f_gas2, f_gas3


def heat_supply(args):
    try:
        price_gas = float(args.get("p_g") or 0.101325)
        price_heat = float(args.get("p_h") or 0.101325)
        price_power = float(args.get("p_p") or 0.101325)
        固定成本 = float(args.get("p1") or 0)
        折旧成本 = float(args.get("p2") or 0)
        power1, power2, power3, f_gas1, f_gas2, f_gas3 = read_data()
        get_f_by_power = fit_curve("燃气流量vs纯凝功率")
        get_power_by_f = fit_curve("纯凝功率vs燃气流量")
        f_gas_供电 = get_f_by_power(power1)
        f_gas_供热 = f_gas1 - f_gas_供电

    except:
        return {}


def start_server():
    # run()  # 启动定时执行
    heat_supply({})
    start_server_app(deal=deal, port=5000,
                     example_url=example)


if __name__ == "__main__":
    example = "http://127.0.0.1:5000/?Action=NaturalGas&CH4=0.98&N2=0.02&T=15&P=3.8&T0=20"
    start_server()



import time

import win32con
import win32gui
import win32api
from ctypes import windll, byref
from ctypes.wintypes import HWND, POINT
import string

VkCode = {
    "back"     : 0x08,
    "tab"      : 0x09,
    "return"   : 0x0D,
    "shift"    : 0x10,
    "control"  : 0x11,
    "menu"     : 0x12,
    "pause"    : 0x13,
    "capital"  : 0x14,
    "escape"   : 0x1B,
    "space"    : 0x20,
    "end"      : 0x23,
    "home"     : 0x24,
    "left"     : 0x25,
    "up"       : 0x26,
    "right"    : 0x27,
    "down"     : 0x28,
    "print"    : 0x2A,
    "snapshot" : 0x2C,
    "insert"   : 0x2D,
    "delete"   : 0x2E,
    "lwin"     : 0x5B,
    "rwin"     : 0x5C,
    "numpad0"  : 0x60,
    "numpad1"  : 0x61,
    "numpad2"  : 0x62,
    "numpad3"  : 0x63,
    "numpad4"  : 0x64,
    "numpad5"  : 0x65,
    "numpad6"  : 0x66,
    "numpad7"  : 0x67,
    "numpad8"  : 0x68,
    "numpad9"  : 0x69,
    "multiply" : 0x6A,
    "add"      : 0x6B,
    "separator": 0x6C,
    "subtract" : 0x6D,
    "decimal"  : 0x6E,
    "divide"   : 0x6F,
    "f1"       : 0x70,
    "f2"       : 0x71,
    "f3"       : 0x72,
    "f4"       : 0x73,
    "f5"       : 0x74,
    "f6"       : 0x75,
    "f7"       : 0x76,
    "f8"       : 0x77,
    "f9"       : 0x78,
    "f10"      : 0x79,
    "f11"      : 0x7A,
    "f12"      : 0x7B,
    "numlock"  : 0x90,
    "scroll"   : 0x91,
    "lshift"   : 0xA0,
    "rshift"   : 0xA1,
    "lcontrol" : 0xA2,
    "rcontrol" : 0xA3,
    "lmenu"    : 0xA4,
    "rmenu"    : 0XA5
}
PostMessageW = windll.user32.PostMessageW
MapVirtualKeyW = windll.user32.MapVirtualKeyW
VkKeyScanA = windll.user32.VkKeyScanA
ClientToScreen = windll.user32.ClientToScreen
WM_MOUSEMOVE = 0x0200
WM_LBUTTONDOWN = 0x0201
WM_LBUTTONUP = 0x202
WM_MOUSEWHEEL = 0x020A
WHEEL_DELTA = 120
WM_KEYDOWN = 0x100
WM_KEYUP = 0x101


def click_key(handle: HWND, key: str):
    """
    按下指定按键，测试成功

    Args:
        handle (HWND): 窗口句柄
        key (str): 按键名
    """
    vk_code = get_virtual_keycode(key)
    scan_code = MapVirtualKeyW(vk_code, 0)
    # https://docs.microsoft.com/en-us/windows/win32/inputdev/wm-keydown
    wparam = vk_code
    lparam = (scan_code << 16) | 1
    PostMessageW(handle, WM_KEYDOWN, wparam, lparam)
    time.sleep(0.5)
    lparam = (scan_code << 16) | 0XC0000001
    PostMessageW(handle, WM_KEYUP, wparam, lparam)


def get_virtual_keycode(key: str):
    """根据按键名获取虚拟按键码

    Args:
        key (str): 按键名

    Returns:
        int: 虚拟按键码
    """
    if len(key) == 1 and key in string.printable:
        # https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-vkkeyscana
        return VkKeyScanA(ord(key)) & 0xff  # ==ord(key)
    else:
        return VkCode[key]


import sys

if not windll.shell32.IsUserAnAdmin():
    # 不是管理员就提权
    windll.shell32.ShellExecuteW(
        None, "runas", sys.executable, __file__, None, 1)



import os

import torch

from yangke.common.config import logger
from yangke.common.qt import YkWindow, run_app, YkItem, YkInputPanel, YkConsole
from yangke.common.QtImporter import QMessageBox, QFileDialog, QHBoxLayout, QCheckBox, QComboBox, QWidget, \
    QLineEdit, QApplication, QStringListModel
from yangke.common.fileOperate import read_csv_ex
from yangke.pytorch.mytorch import DataSetFitting, DataFitterNet
from yangke.pytorch.ui.PriorLoss_PyQt5 import Ui_Form


class PriorLossWidget(QWidget):
    def __init__(self):
        super(PriorLossWidget, self).__init__()
        self.total_items = 10
        self.ui = Ui_Form()
        self.ui.setupUi(self)
        self.x_titles = []
        self.y_title = None
        self.rules = []
        # self.ui.btn_ok.clicked.connect(self.save_rules)  btn_ok的点击事件有根组件完成，因为要保存信息至项目级别
        for i in range(1, self.total_items, 1):
            exec(f"self.ui.check{i}.stateChanged.connect(lambda: self.checked_changed({i}))", {"self": self})

    def update_parameters(self, x_titles, y_title):
        self.x_titles = x_titles
        self.y_title = y_title

    def checked_changed(self, idx):
        ui = self.ui
        check: QCheckBox = eval(f"ui.check{idx}")
        condition: QHBoxLayout = eval(f"ui.condition{idx}")
        widget: QWidget = eval(f"ui.widget{idx}")
        layout: QHBoxLayout = widget.layout()
        if check.isChecked():
            combo1: QComboBox = getattr(ui, f"prior{idx}1")
            combo2: QComboBox = getattr(ui, f"prior{idx}2")
            combo3: QComboBox = getattr(ui, f"prior{idx}3")
            combo4: QComboBox = getattr(ui, f"prior{idx}4")
            layout.removeItem(condition)
            layout.removeWidget(combo4)
            condition.deleteLater()
            combo1.deleteLater()
            combo2.deleteLater()
            combo3.deleteLater()
            combo4.deleteLater()
            QApplication.processEvents()
            del combo4
            del condition
            condition = QLineEdit()
            layout.addWidget(condition)
            setattr(ui, f"condition{idx}", condition)
        else:
            layout.removeWidget(condition)
            condition.deleteLater()
            QApplication.processEvents()
            condition = QHBoxLayout()
            combo1 = QComboBox()
            combo2 = QComboBox()
            combo3 = QComboBox()
            combo4 = QComboBox()
            condition.addWidget(combo1)
            condition.addWidget(combo2)
            condition.addWidget(combo3)
            setattr(ui, f"prior{idx}1", combo1)
            setattr(ui, f"prior{idx}2", combo2)
            setattr(ui, f"prior{idx}3", combo3)
            setattr(ui, f"prior{idx}4", combo4)
            setattr(ui, f"condition{idx}", condition)
            combo4.setDisabled(True)
            layout.addLayout(condition)
            layout.addWidget(combo4)
        self.display_rules(idx=idx)

    def display_rules(self, rules: dict | None = None, idx=None):
        """
        将规则显示在PriorLossWidget上
        :param rules:
        :param idx: 设置单个rule时，指定rule的索引，索引从1开始
        :return:
        """
        if rules is None:
            rules = [None] * self.total_items

        exp_list = ["无"]
        if self.x_titles is None:
            return
        for x in self.x_titles:
            exp_list.append(f"[{self.y_title[0]}]随[{x}]单调递增")
            exp_list.append(f"[{self.y_title[0]}]随[{x}]单调递减")
        for i in range(self.total_items):
            if i < len(rules):
                rule = rules[i]
            else:
                rule = None

            check: QCheckBox = getattr(self.ui, f"check{i + 1}")
            if idx is not None:
                if i + 1 != idx:
                    continue
            if rule is None:
                if check.isChecked():
                    ...
                else:
                    combo1: QComboBox = getattr(self.ui, f"prior{i + 1}1")
                    combo2: QComboBox = getattr(self.ui, f"prior{i + 1}2")
                    combo3: QComboBox = getattr(self.ui, f"prior{i + 1}3")
                    combo4: QComboBox = getattr(self.ui, f"prior{i + 1}4")
                    combo2.setModel(QStringListModel(["无", "大于", "等于", "小于", "任何条件"]))
                    combo2.setCurrentIndex(0)
                    combo1.hide()
                    combo3.hide()
                    combo4.setModel(QStringListModel(exp_list))
                    exec(f"combo2.currentTextChanged.connect(lambda: self.condition_changed(combo2, {i + 1}))",
                         {"self": self, f"combo2": combo2})

            else:
                is_checked = rule.get("isChecked")
                if is_checked:
                    check.setChecked(True)
                    condition: QLineEdit = getattr(self.ui, f"condition{i + 1}")
                    condition.setText(rule.get("expression") or "")
                else:
                    check.setChecked(False)
                    var1 = rule.get("var1")
                    op = rule.get("op")
                    var2 = rule.get("var2")
                    expression = rule.get("expression")
                    combo1: QComboBox = getattr(self.ui, f"prior{i + 1}1")
                    combo2: QComboBox = getattr(self.ui, f"prior{i + 1}2")
                    combo3: QComboBox = getattr(self.ui, f"prior{i + 1}3")
                    combo4: QComboBox = getattr(self.ui, f"prior{i + 1}4")
                    exec(f"combo2.currentTextChanged.connect(lambda: self.condition_changed(combo2, {i + 1}))",
                         {"self": self, f"combo2": combo2})

                    combo1.setModel(QStringListModel(self.x_titles))
                    combo2.setModel(QStringListModel(["无", "大于", "等于", "小于", "任何条件"]))
                    combo3.setModel(QStringListModel(self.x_titles))
                    combo4.setModel(QStringListModel(exp_list))
                    combo1.setCurrentText(var1)
                    combo2.setCurrentText(op)
                    combo3.setCurrentText(var2)
                    combo4.setCurrentText(expression)

    def condition_changed(self, combox, idx=None):
        text = combox.currentText()
        combo1: QComboBox = getattr(self.ui, f"prior{idx}1")
        combo2: QComboBox = getattr(self.ui, f"prior{idx}2")
        combo3: QComboBox = getattr(self.ui, f"prior{idx}3")
        combo4: QComboBox = getattr(self.ui, f"prior{idx}4")
        model1 = QStringListModel(self.x_titles)
        model3 = QStringListModel(self.x_titles)
        combo1.setModel(model1)
        combo3.setModel(model3)
        if text in ["大于", "等于", "小于"]:
            combo1.show()
            combo3.show()
            combo4.setDisabled(False)
        elif text == "无":
            combo1.hide()
            combo3.hide()
            combo4.setCurrentText("无")
            combo4.setDisabled(True)
        else:
            combo1.hide()
            combo3.hide()
            combo4.setDisabled(False)

    def parse_prior_loss_rules(self):
        """
        从PriorLossWidget面板上获取并返回PriorLoss的设置信息

        :return:
        """
        self.rules = []
        for i in range(self.total_items):
            check: QCheckBox = getattr(self.ui, f"check{i + 1}")
            if check.isChecked():
                exp_lineedit: QLineEdit = getattr(self.ui, f"condition{i + 1}")
                exp = exp_lineedit.text()
                self.rules.append({"isChecked": True, "expression": exp})
            else:
                combo1: QComboBox = getattr(self.ui, f"prior{i + 1}1")
                combo2: QComboBox = getattr(self.ui, f"prior{i + 1}2")
                combo3: QComboBox = getattr(self.ui, f"prior{i + 1}3")
                combo4: QComboBox = getattr(self.ui, f"prior{i + 1}4")
                var1 = combo1.currentText()
                op = combo2.currentText()
                var2 = combo3.currentText()
                exp = combo4.currentText()

                if op == "无":
                    continue
                else:
                    self.rules.append({"isChecked": False, "var1": var1, "op": op,
                                       "var2": var2, "expression": exp})
        return self.rules


class MainWindow(YkWindow):
    def __init__(self):
        super(MainWindow, self).__init__()  # 完成了通用变量的加载
        self.setWindowTitle("数据分析")
        self.df = None  # 声明变量
        self.file = None  # 声明变量
        self.dataset = None  # 声明变量
        self.net = None
        self.display_project(self.proj)  # 展示本应用的数据至画面
        self.model = None
        self.console = YkConsole()
        self.add_dock_widget('终端', self.console, "bottom")
        logger.add(self.console)

    def open(self, proj=None):
        super(MainWindow, self).open(proj)  # 父类加载项目信息至self.proj中
        if isinstance(self.proj, dict):
            self.display_project(self.proj)  # 子类显示self.proj信息

    def new_project(self):
        super(MainWindow, self).new_project()
        self.display_project(self.proj)

    def set_predict_method(self, method):
        if method == "svm":
            self.proj.update({"predict_method": "svm"})
        else:
            self.proj.update({"predict_method": "nn"})
        self.display_project()

    def display_project(self, proj=None):
        """
        展示数据至画面

        :param proj:
        :return:
        """
        self.enable_table("ui/ui_panel.yaml")
        self.enable_input_panel("ui/ui_panel.yaml", domain="panel")
        _predict_method = self.proj.get("predict_method")
        if _predict_method is None or _predict_method == "nn":
            if self._input_tab is None or "set_nn" not in self._input_tab.labels:
                self.add_input_panel("ui/ui_panel.yaml", domain="set_nn")
            if "set_svm" in self._input_tab.labels:
                self._input_tab.remove_tab("set_svm")
        else:
            if self._input_tab is None or "set_svm" not in self._input_tab.labels:
                self.add_input_panel("ui/ui_panel.yaml", domain="set_svm")
            if "set_nn" in self._input_tab.labels:
                self._input_tab.remove_tab("set_nn")
        self.add_input_panel("ui/ui_panel.yaml", domain="set_var")
        self.add_input_panel(domain="set_predict")
        # 添加计算结果面板后，会导致self._table_widget面板不显示dataframe数据，原因未知
        # self.add_content_tab(YkDataTableWidget(from_file=None), "计算结果", replace=False)
        if self.proj == {}:
            # 清空软件界面上现有的数据
            self.replace_table("ui/ui_table.yaml")
            self.replace_input_panel("ui/ui_panel.yaml", domain="panel")
        elif self.proj is not None and self.proj is not False:
            self.file = self.proj.get("datafile")
            if self.file is None:
                return
            self.df = self.proj.get("dataframe")
            self._input_tab.activate_tab("panel")
            self.set_value_of_panel({"数据文件": self.file}, panel="panel")
            if os.path.exists(self.file):
                self.df = read_csv_ex(self.file)
            if self.df is None:
                return
            self._table_widget.display_dataframe(self.df[:1000], index=None)
            # 清空所有已添加的输入参数
            self._input_panel = self._input_tab.get_tab_panel("panel")
            while len(self._input_panel.get_values_and_units(need_unit=False)) > 1:
                self._input_panel.remove_item(index=1)
            #
            title = list(self.df.columns)
            items = [
                YkItem(label="选择神经网络输入参数：", size=[200, 50, 50]),
                YkItem(label="input 1", value=title, size=[50, 150, 100]),
                YkItem(label="", value='<button on-click="remove_input_item()">删除输入参数</button>',
                       unit='<button on-click="insert_input_item()">添加输入参数</button>', size=[50, 100, 100]),

                YkItem(label="选择神经网络输出参数：", size=[200, 50, 50]),
                YkItem(label="output 1", value=title, size=[50, 150, 100]),
                YkItem(label="", value='<button on-click="remove_output_item()">删除输出参数</button>',
                       unit='<button on-click="insert_output_item()">添加输出参数</button>', size=[50, 100, 100]),
            ]
            self._input_panel = self._input_tab.get_tab_panel("panel")
            self._input_panel.append_item(items)
            if self.proj.get("x_titles") is not None:
                x_titles = self.proj.get("x_titles")
                if len(x_titles) > 0:
                    self._input_panel.set_value("input 1", x_titles[0])
                for i in range(1, len(x_titles)):
                    self.insert_input_item()
                    self._input_panel.set_value(f"input {i + 1}", x_titles[i])
            if self.proj.get("y_titles") is not None:
                y_titles = self.proj.get("y_titles")
                if len(y_titles) > 0:
                    self._input_panel.set_value("output 1", y_titles[0])
                for i in range(1, len(y_titles)):
                    self.insert_output_item()
                    self._input_panel.set_value(f"output {i + 1}", y_titles[i])
            self.proj.update({"dataframe": self.df})

            # 更新set_nn输入面板
            if _predict_method == "nn":
                self._input_panel = self._input_tab.get_tab_panel("set_nn")
                self._input_panel.set_value("损失函数类型", self.proj.get("loss_func"))
                self._input_panel.set_value("优化器", self.proj.get("optimizer"))
                self._input_panel.set_value("学习率", self.proj.get("learn_rate"))
                self._input_panel.set_value(label="训练轮次数", value=self.proj.get("epochs"))
                self._input_panel.set_value(label="批大小", value=self.proj.get("batch_size"))
                self._input_panel.set_value("神经网络保存路径", self.proj.get("save_path"))
                self._input_panel.set_value("训练集比例", self.proj.get("data_training_proportion"))
                self._input_panel.set_value("测试集比例", self.proj.get("data_test_proportion"))

                self.change_normal_method(self.proj.get("normalization_method"))

                self._input_panel.set_value("数据归一化方法", self.proj.get("normalization_method"))
            else:
                self._input_panel = self._input_tab.get_tab_panel("set_svm")
                self._input_panel.set_value("核", self.proj.get("svm_kernel"))

            combo_box = self._input_tab.get_tab_panel("set_var").get_item("数据归一化方法").value
            combo_box.currentTextChanged.connect(self.change_normal_method)

        self._input_tab.activate_tab("panel")

    def change_normal_method(self, sender):
        self._input_panel: YkInputPanel = self._input_tab.get_tab_panel("set_var")
        if self.proj.get("x_titles") is None:
            return
        titles = self.proj.get("x_titles").copy()
        titles.extend(self.proj.get("y_titles"))
        labels, _ = self._input_panel.get_values_and_units(need_label=True, need_dict=False, need_unit=False)

        _vars = self.proj.get("v_range") or {}

        for label in labels:  # 移除面板上已有的数据
            if label.startswith("参数:"):
                self._input_panel.remove_item(name=label)
        if sender == "min-max":
            items = []
            for title in titles:
                up, bot = _vars.get(title) or (0, 0)
                items.append(
                    YkItem(f"参数:{title}", up, f"<input type='text' value='{bot}'>", size=[100, 100, 100], indent=10))
            self._input_panel.append_item(items)
        else:
            pass

    def start_train(self):
        """
        开始神经网络训练

        :return:
        """
        if self.proj.get("x_titles") is None:
            msg = "error: no dataset!"
            QMessageBox.information(self, "提示信息", "数据集未设置！！")
            return msg
        device = "cuda" if torch.cuda.is_available() else "cpu"
        self.proj.update({"device": device})
        if self.proj.get("epochs") is None:
            QMessageBox.information(self, "提示信息", "神经网络未设置，切换到神经网络设置面板！")
            self._input_tab.activate_tab("set_nn")
            return

        # 构建神经网络数据集
        classify = True if self.proj.get('loss_func') == "CrossEntropy" else False
        self.dataset = DataSetFitting(self.df, x_title=self.proj.get("x_titles"), y_title=self.proj.get("y_titles"),
                                      proportion_train=float(self.proj.get("data_training_proportion")),
                                      proportion_test=float(self.proj.get("data_test_proportion")),
                                      classify=True)
        if classify:
            y_class_num = self.df[self.proj.get('y_titles')].drop_duplicates()  # 分类的类别数量
            y_class_num = list(y_class_num.to_numpy().squeeze())
            y_class_num = [int(_) for _ in y_class_num]
            if 0 not in y_class_num:
                logger.warning(f"当前为分类问题，分类问题中，类别编号必须以0开始，但当前类别标签中未发现类别为0的数据项")
            y_class_num = len(y_class_num)
            bak_info = y_class_num
        else:
            bak_info = self.proj.get("loss_func_bak_info")
        if self.proj.get("normalization_method") == "min-max":
            v_range = self.proj.get("v_range")
            x_range = [v_range[x] for x in self.proj.get("x_titles")]
            y_range = [v_range[y] for y in self.proj.get("y_titles")]
            self.dataset.set_standard(normal_method=self.proj.get("normalization_method"),
                                      x_range=x_range, y_range=y_range)
        else:
            self.dataset.set_standard(normal_method="z-score")

        settings = {
            "input": self.proj.get("x_titles"),
            "output": self.proj.get("y_titles"),
            "classify": classify,
            "networks": {
                "train": {
                    "epochs": self.proj.get("epochs"),
                    "batch_size": self.proj.get("batch_size"),
                    "learning_rate": self.proj.get("learn_rate"),
                },
                "loss_fn": {
                    "type": self.proj.get("loss_func"),
                    "bak_info": bak_info
                },
                "optimizer": {
                    "type": self.proj.get("optimizer")
                },
                "layers": self.proj.get("layers"),
            },
            "save_path": self.proj.get("save_path"),
            "normalization_method": self.proj.get("normalization_method")
        }

        # 构建神经网络对象
        self.net = DataFitterNet(settings1=settings)
        model = self.net.to(device)  # 将神经网络迁移到cpu或gpu上
        model.start_train(self.dataset)
        model.save_yk(self.proj.get("save_path"))
        self.model = model

    def update_proj(self):
        """
        更新项目信息

        :return:
        """
        # noinspection all
        self._input_panel: YkInputPanel = self._input_tab.currentWidget()
        _l, _v, _u = self._input_panel.get_values_and_units(need_unit=True, need_dict=False,
                                                            need_label=True)  # 不能使用dict，因为有重名
        x_titles = []
        y_titles = []
        layers = []
        v_range = {}
        for k, v, u in zip(_l, _v, _u):
            if k.startswith('input '):
                x_titles.append(v)
            elif k.startswith('output '):
                y_titles.append(v)
            elif k.startswith("损失函数"):
                self.proj["loss_func"] = v
            elif k.startswith("数据归一化方法"):
                self.proj["normalization_method"] = v
            elif k == "优化器":
                self.proj["optimizer"] = v
            elif k == "学习率":
                self.proj["learn_rate"] = float(v)
            elif k == "训练轮次数":
                self.proj["epochs"] = int(v)
            elif k == "批大小":
                self.proj["batch_size"] = int(v)
            elif k == "神经网络保存路径":
                self.proj["save_path"] = v
            elif k == "训练集比例":
                self.proj["data_training_proportion"] = float(v)
            elif k == "测试集比例":
                self.proj["data_test_proportion"] = float(v)
            elif k == "输入层":
                layers.append({"type": "input", "cell_num": "auto"})
            elif k.startswith("隐藏层"):
                _temp = k
                layers.append({})
            elif k == "类型":
                layers[-1].update({"type": v})
            elif k == "神经元数量":
                # noinspection all
                layers[-1].update({"cell_num": int(v)})
            elif k == "偏置":
                bias = True if v == "是" else False
                # noinspection all
                layers[-1].update({"bias": bias})
            elif k == "输出层":
                layers.append({"type": "output", "cell_num": "auto"})
            elif k.startswith("参数:"):
                k = k.replace("参数:", "")
                v_range.update({k: [float(v), float(u)]})
            elif k == "核":
                self.proj.update({"svm_kernel": v})
            elif k == "gamma":
                self.proj.update({"svm_gamma": v})
            elif k == "阶数":
                self.proj.update({"svm_degree": v})

        if len(x_titles) > 0:
            self.proj.update({"x_titles": x_titles, "y_titles": y_titles})
        if len(layers) > 0:
            self.proj.update({"layers": layers})
        if len(v_range) > 0:
            self.proj.update({"v_range": v_range})

    def choose_predict_file(self):
        files, _ = QFileDialog.getOpenFileName(self, '选择数据文件', os.getcwd(), "All Files(*)")
        if files is None or files == "":
            return
        if not files.endswith(".csv"):
            logger.warning(f"选择的文件{files}不是正确的数据文件，请检查!")
            return
        self._input_panel = self._input_tab.get_tab_panel("set_predict")
        self.set_value_of_panel({"数据文件": files})

    def predict(self):
        """
        加载模型并预测
        :return:
        """
        table_widget = self._content_tab.get_tab_panel("计算结果")
        self._input_panel = self._input_tab.get_tab_panel("set_predict")
        file = self._input_panel.get_item("数据文件").get_value()
        out_file = self._input_panel.get_item("预测结果输出文件").get_value()
        if not os.path.exists(file):
            QMessageBox.information(self, "提示信息", "数据文件不存在！")
            return
        df = read_csv_ex(file)
        if self.model is None:
            self.model = DataFitterNet.load_yk(self.proj.get("save_path"))
        y = self.model.prediction(df)
        y = y.flatten()
        df["predict"] = y
        if not os.path.exists(os.path.dirname(os.path.abspath(out_file))):
            os.makedirs(os.path.dirname(os.path.abspath(out_file)))

        df.to_excel(out_file, index=False)
        logger.debug(f"预测结果输出值{os.path.abspath(out_file)}")

    def choose_file(self):
        files, _ = QFileDialog.getOpenFileName(self, '选择数据文件', os.getcwd(), "All Files(*)")
        if files is None or files == "":
            return
        if not files.endswith(".csv"):
            logger.warning(f"选择的文件{files}不是正确的数据文件，请检查!")
            return
        self._input_panel = self._input_tab.get_tab_panel("panel")
        self.set_value_of_panel({"数据文件": files})
        value = self.get_value_of_panel(need_unit=False, need_dict=True)
        self.statusBar1.showMessage(str(value))
        self.file = files
        self.proj.update({"datafile": files})
        self.display_project(self.proj)

    def read_data(self):
        files = self.get_value_of_panel(need_unit=False, need_dict=True)['数据文件']
        if isinstance(files, list):
            for file in files:
                if os.path.exists(file):
                    self.df = read_csv_ex(file)
        else:
            if os.path.exists(files):
                self.df = read_csv_ex(files)
        self.enable_table()
        if self.df is None:
            return
        self._table_widget.display_dataframe(self.df)
        title = list(self.df.columns)
        items = [
            YkItem(label="选择神经网络输入参数：", size=[200, 50, 50]),
            YkItem(label="input 1", value=title, size=[50, 150, 100]),
            YkItem(label="", value='<button on-click="remove_input_item()">删除输入参数</button>',
                   unit='<button on-click="insert_input_item()">添加输入参数</button>', size=[50, 100, 100]),

            YkItem(label="选择神经网络输出参数：", size=[200, 50, 50]),
            YkItem(label="output 1", value=title, size=[50, 150, 100]),
            YkItem(label="", value='<button on-click="remove_output_item()">删除输出参数</button>',
                   unit='<button on-click="insert_output_item()">添加输出参数</button>', size=[50, 100, 100]),
        ]
        self._input_panel.append_item(items)
        self.proj.update({"dataframe": self.df})

    def insert_input_item(self, value=None):
        self._input_panel = self._input_tab.get_tab_panel("panel")
        values = self.get_value_of_panel(need_dict=True, need_unit=False)
        input_values = [e for e in values if e.startswith("input")]
        title = list(self.df.columns)
        item = YkItem(label=f"input {len(input_values) + 1}", value=title, size=[50, 150, 100])
        self._input_panel.insert_item(len(input_values) + 2, item)

    def remove_input_item(self):
        self._input_panel = self._input_tab.get_tab_panel("panel")
        values = self.get_value_of_panel(need_dict=True, need_unit=False)
        input_values = [e for e in values if e.startswith("input")]
        if len(input_values) <= 1:
            logger.warning(f"输入变量至少应有1个，无法继续删除")
            return
        self._input_panel.remove_item(name=input_values[-1])

    def insert_output_item(self):
        self._input_panel = self._input_tab.get_tab_panel("panel")
        values: dict = self.get_value_of_panel(need_dict=True, need_unit=False)
        output_values = [k for k, v in values.items() if k is not None and k.startswith("output")]
        title = list(self.df.columns)
        item = YkItem(label=f"output {len(output_values) + 1}", value=title, size=[50, 150, 100])
        idx = list(values.keys()).index(output_values[-1]) + 1
        self._input_panel.insert_item(idx, item)

    def remove_output_item(self):
        self._input_panel = self._input_tab.get_tab_panel("panel")
        values = self.get_value_of_panel(need_dict=True, need_unit=False)
        output_names = [e for e in values if e.startswith("output")]
        self._input_panel.remove_item(name=output_names[-1])

    def loss_changed(self):
        """
        修改损失函数

        :return:
        """
        current_input_panel_name = self._input_panel.name
        self._input_panel = self._input_tab.get_tab_panel("set_nn")
        loss_type = self._input_panel.get_item("损失函数类型").get_value()
        if loss_type == "PriorLoss":
            widget = PriorLossWidget()
            widget.update_parameters(x_titles=self.proj.get("x_titles"), y_title=self.proj.get("y_titles"))
            widget.display_rules(self.proj.get("loss_func_bak_info"))
            if self._content_tab is None:
                self.add_content_tab(widget=widget, tab_name="设置PriorLoss函数", replace=False)
            a0 = self._content_tab.widget(0).geometry()
            widget.setGeometry(a0)
            self.add_content_tab(widget=widget, tab_name="设置PriorLoss函数", replace=False)
            self._content_tab.activate_tab('设置PriorLoss函数')
            widget.ui.btn_ok.clicked.connect(self.save_prior_loss_info)
        else:
            if loss_type == "CrossEntropy":
                self.statusBar1.showMessage("CrossEntropy为多分类问题的损失函数！")
                logger.debug(
                    f"CrossEntropy为多分类问题的损失函数，分类问题中，输出参数必须为[0, 1, 2, ..., n]等代表类别的整数")
                logger.debug(f"选择该损失函数，模型将自动切换为分类预测神经网络！")
            else:
                ...
            if self._content_tab is None:
                return
            widget = self._content_tab.get_tab_panel("设置PriorLoss函数")
            if widget is not None:
                self._content_tab.removeTab(self._content_tab.indexOf(widget))

        self._input_panel = self._input_tab.get_tab_panel(current_input_panel_name)

    def save_prior_loss_info(self):
        # noinspection all
        widget: PriorLossWidget = self._content_tab.get_tab_panel("设置PriorLoss函数")
        if widget is None:
            return
        rules = widget.parse_prior_loss_rules()
        self.proj.update({"loss_func_bak_info": rules})  # 更新损失函数的备注信息
        self._content_tab.activate_tab('sheet 1')


if __name__ == "__main__":
    run_app(MainWindow)



import pandas as pd
import numpy as np

from yangke.common.config import logger
from yangke.sis.榆能横山.tags import (get_backpressure, get_opt_type, TagsRead, TagsWrite, load_data,
                                  get_flowrate, get_pump_power, get_dpower_per_kpa, get_coal_consume, )
from yangke.dataset.PIDB import PIDB
from yangke.sis.dll_file import get_tag_value, ColdResult
from yangke.base import execute_function_by_interval
import datetime

_ = load_data(r"D:\LDYH\冷端优化软件输入数据.xlsx")
backpressure, type_opt_df, pump_info, coal_info = _
db = PIDB()
cold_result = ColdResult()
cold_result.设置背压标签(["最优运行方式下背压1", "最优运行方式下背压2", "优化后机组节煤量1",
                    "优化后机组节煤量2"])  # 背压标签的值不会受到cold_result.平抑波动()方法影响
allowed_range = [("power1", "±10"), ("temp1", "±4"), ("power2", "±10"), ("temp2", "±2")]  # 1、2号机的参数分别判断
cold_result.设置允许的运行参数波动范围(allowed_range)
计时器1号机 = {}
计时器2号机 = {}


def get_average_value(values):
    """
    求数组的平均值，并希望能删除明显偏离正常值的错误数据

    :param values:
    :return:
    """
    if isinstance(values, list):
        values = np.array(values)
        average = values.mean()
        return average


def get_pump_type_str(pump1_I, pump4_I, pump2_IH, pump2_IL, pump3_IH, pump3_IL, need_detail=False):
    高速台数 = 0
    低速台数 = 0
    detail = {}
    if pump1_I > 10:
        高速台数 = 高速台数 + 1
        detail.update({1: "高速"})
    else:
        detail.update({1: "关"})
    if pump4_I > 10:
        高速台数 += 1
        detail.update({4: "高速"})
    else:
        detail.update({4: "关"})
    if pump2_IH > 10:
        高速台数 += 1
        detail.update({2: "高速"})
    elif pump2_IL > 10:
        低速台数 += 1
        detail.update({2: "低速"})
    else:
        detail.update({2: "关"})
    if pump3_IH > 10:
        高速台数 += 1
        detail.update({3: "高速"})
    elif pump3_IL > 10:
        低速台数 += 1
        detail.update({3: "低速"})
    else:
        detail.update({3: "关"})

    if (高速台数, 低速台数) == (4, 0):
        type_ = "四台高速"
    elif (高速台数, 低速台数) == (3, 1):
        type_ = "三高一低"
    elif (高速台数, 低速台数) == (2, 2):
        type_ = "两高两低"
    elif (高速台数, 低速台数) == (3, 0):
        type_ = "三台高速"
    elif (高速台数, 低速台数) == (2, 1):
        type_ = "两高一低"
    elif (高速台数, 低速台数) == (1, 2):
        type_ = "一高两低"
    elif (高速台数, 低速台数) == (2, 0):
        type_ = "两台高速"
    elif (高速台数, 低速台数) == (1, 1):
        type_ = "一高一低"
    elif (高速台数, 低速台数) == (0, 2):
        type_ = "两台低速"
    elif (高速台数, 低速台数) == (0, 1):
        type_ = "一台低速"
    elif (高速台数, 低速台数) == (0, 0):
        type_ = "泵全停"
    elif (高速台数, 低速台数) == (1, 0):
        type_ = "一台高速"
    else:
        type_ = "未知方式"
    if need_detail:
        return type_, detail
    else:
        return type_


def get_pump_type_int(type_: str):
    if type_ == "四台高速":
        return 4, 0
    elif type_ == "三高一低":
        return 3, 1
    elif type_ == "两高两低":
        return 2, 2
    elif type_ == "三台高速":
        return 3, 0
    elif type_ == "两高一低":
        return 2, 1
    elif type_ == "一高两低":
        return 1, 2
    elif type_ == "两台高速":
        return 2, 0
    elif type_ == "一高一低":
        return 1, 1
    elif type_ == "两台低速":
        return 0, 2
    elif type_ == "一台低速":
        return 0, 1
    elif type_ == "泵全停":
        return 0, 0
    elif type_ == "一台高速":
        return 1, 0
    else:
        return 0, 0


def cal_group(unit, snapshot):
    power = get_tag_value(snapshot, f"发电机功率{unit}", 0)
    pump1_I = get_tag_value(snapshot, f"机组{unit}循环水泵1电流", 0)
    pump4_I = get_tag_value(snapshot, f"机组{unit}循环水泵4电流", 0)  # 1、4为定速泵
    pump2_IL = get_tag_value(snapshot, f"机组{unit}循环水泵2低速电流", 0)
    pump2_IH = get_tag_value(snapshot, f"机组{unit}循环水泵2高速电流", 0)
    pump3_IL = get_tag_value(snapshot, f"机组{unit}循环水泵3低速电流", 0)
    pump3_IH = get_tag_value(snapshot, f"机组{unit}循环水泵3高速电流", 0)
    type_, detail = get_pump_type_str(pump1_I, pump4_I, pump2_IH, pump2_IL, pump3_IH, pump3_IL, need_detail=True)
    changed = []
    if unit == 1:
        计时器 = 计时器1号机
    else:
        计时器 = 计时器2号机
    if 计时器.get("last_state") is None:  # 初始状态
        计时器.update({"last_state_time": datetime.datetime.now(),
                    "last_state": detail, "initial_state": True})
        计时器.update({"last_state_time1": datetime.datetime.now(), "last_state1": detail.get(1)})
        计时器.update({"last_state_time2": datetime.datetime.now(), "last_state2": detail.get(2)})
        计时器.update({"last_state_time3": datetime.datetime.now(), "last_state3": detail.get(3)})
        计时器.update({"last_state_time4": datetime.datetime.now(), "last_state4": detail.get(4)})
    elif 计时器.get("last_state") == detail:  # 状态未发生变化
        pass
    else:
        # 说明机组冷端运行方式发生变化
        last_detail = 计时器.get("last_state")
        detail1 = last_detail[1]
        detail2 = last_detail[2]
        detail3 = last_detail[3]
        detail4 = last_detail[4]
        if detail1 != detail.get(1):
            changed.append(1)
            计时器.update({"last_state_time1": datetime.datetime.now(), "last_state1": detail.get(1)})
        if detail2 != detail.get(2):
            changed.append(2)
            计时器.update({"last_state_time2": datetime.datetime.now(), "last_state2": detail.get(2)})
        if detail3 != detail.get(3):
            changed.append(3)
            计时器.update({"last_state_time3": datetime.datetime.now(), "last_state3": detail.get(3)})
        if detail4 != detail.get(4):
            changed.append(4)
            计时器.update({"last_state_time4": datetime.datetime.now(), "last_state4": detail.get(4)})

        计时器.update({"last_state_time": datetime.datetime.now(),
                    "last_state": detail, "initial_state": False})
    # elif unit == 2:
    #     if 计时器2号机.get("last_state") is None:
    #         计时器2号机.update({"last_state_time": datetime.datetime.now(),
    #                        "last_state": type_, "initial_state": True})
    #     elif 计时器2号机.get("last_state") == type_:
    #         pass
    #     elif 计时器2号机.get("last_state") != type_:
    #         # 说明1号机组冷端运行方式发生变化
    #         计时器2号机.update({"last_state_time": datetime.datetime.now(),
    #                        "last_state": type_, "initial_state": False})

    # temp0 = get_tag_value(snapshot, f"机组{unit}环境温度1", 0)
    # temp1 = get_tag_value(snapshot, f"机组{unit}环境温度2", 0)
    # temp2 = get_tag_value(snapshot, f"机组{unit}环境温度3", 0)
    # temp3 = get_tag_value(snapshot, f"机组{unit}环境温度4", 0)
    # temp4 = get_tag_value(snapshot, f"机组{unit}环境温度5", 0)
    # temp5 = get_tag_value(snapshot, f"机组{unit}环境温度6", 0)
    # temp6 = get_tag_value(snapshot, f"机组{unit}环境温度7", 0)
    # temp7 = get_tag_value(snapshot, f"机组{unit}环境温度8", 0)
    # temp8 = get_tag_value(snapshot, f"机组{unit}环境温度9", 0)
    pres0 = get_tag_value(snapshot, f"机组{unit}大气压力1", 95)
    pres1 = get_tag_value(snapshot, f"机组{unit}大气压力2", 95)
    t_cyc_cold_1 = get_tag_value(snapshot, f"机组{unit}低压凝汽器循环水进水温度1", 0)
    t_cyc_cold_2 = get_tag_value(snapshot, f"机组{unit}低压凝汽器循环水进水温度2", 0)
    pres = get_average_value([pres0, pres1])
    bp_true = get_tag_value(snapshot, f"机组{unit}凝汽器真空", 0) + pres
    temp_group = get_average_value([t_cyc_cold_1, t_cyc_cold_2])  # 冷却塔出塔水温平均值

    if power > 300:
        type_opt = get_opt_type(power, temp_group, type_opt_df)
        bp_opt = get_backpressure(power, type_opt, temp_group, backpressure)
        bp_理论值 = get_backpressure(power, type_, temp_group, backpressure)
        pump_power_opt = get_pump_power(type_opt, pump_info)
        pump_power_true = get_pump_power(type_, pump_info)
        flowrate_opt = get_flowrate(type_opt, pump_info)
        _ = get_dpower_per_kpa(power, coal_info)
        出力增加值 = -_ * (bp_opt - bp_理论值)  # 当冷端功率增加时，出力也增加
        冷端功率增加值 = pump_power_opt - pump_power_true
        dpower = 出力增加值 - 冷端功率增加值  # 出力增加值-循泵耗功增加值
        coal_save = dpower / power / 1000 * get_coal_consume(power, coal_info)

        _opt = (type_opt, bp_opt, pump_power_opt, flowrate_opt, coal_save, dpower)
        _true = (power, temp_group, pump_power_true, bp_true, bp_理论值, type_, changed)
    else:  # 停机不运算
        _opt = (0, 0, 0, 0, 0, 0)
        _true = (power, temp_group, 0, 0, bp_true, "泵全停", [])
    return _opt, _true


def transfer_pump_type(type_):
    if type_ == "四台高速":
        return 2, 2, 2, 2
    elif type_ == "三高一低":
        return 2, 2, 1, 2
    elif type_ == "两高两低":
        return 2, 1, 1, 2
    elif type_ == "三台高速":
        return 2, 2, 0, 2
    elif type_ == "两高一低":
        return 2, 0, 1, 2
    elif type_ == "一高两低":
        return 2, 1, 1, 0
    elif type_ == "两台高速":
        return 2, 0, 0, 2
    elif type_ == "一高一低":
        return 2, 0, 1, 0
    elif type_ == "两台低速":
        return 0, 1, 1, 0
    elif type_ == "一台低速":
        return 0, 0, 1, 0
    elif type_ == "一台高速":
        return 2, 0, 0, 0
    else:
        return 0, 0, 0, 0


def time_delta_2_hour_minute(time_delta: datetime.timedelta):
    seconds = time_delta.seconds
    days = time_delta.days
    minutes = seconds // 60  # 总的分钟数
    hours = minutes // 60 + days * 24  # 总的小时数
    minutes = minutes % 60  # 总的分钟数对60求余为剩余的分钟数
    return hours, minutes


def optimize():
    snapshot = db.get_snapshot(tags=TagsRead.get_values(), description=TagsRead.get_keys())
    _opt, _real = cal_group(1, snapshot)
    type_opt1, bp_opt1, pump_power_opt1, flowrate_opt1, coal_save1, dpower1 = _opt
    power1, temp1, pump_power1, bp_true1, bp_理论值1, type1_, changed1 = _real
    _opt, _real = cal_group(2, snapshot)
    type_opt2, bp_opt2, pump_power_opt2, flowrate_opt2, coal_save2, dpower2 = _opt
    power2, temp2, pump_power2, bp_true2, bp_理论值2, type2_, changed2 = _real

    pump11, pump12, pump13, pump14 = transfer_pump_type(type_opt1)
    pump21, pump22, pump23, pump24 = transfer_pump_type(type_opt2)

    opt_res = {
        "type_opt1": type_opt1,
        "pump11_unit1": pump11,  # 分机组平抑波动时，变量名最后一个字符必须与机组编号相同
        "pump12_unit1": pump12,
        "pump13_unit1": pump13,
        "pump14_unit1": pump14,
        "bp_opt1": bp_opt1,
        "pump_power_opt1": pump_power_opt1,
        "flowrate_opt1": flowrate_opt1,
        "coal_save1": coal_save1,
        "dpower1": dpower1,

        "type_opt2": type_opt2,
        "pump21_unit2": pump21,  # 分机组平抑波动时，变量名最后一个字符必须与机组编号相同
        "pump22_unit2": pump22,
        "pump23_unit2": pump23,
        "pump24_unit2": pump24,
        "bp_opt2": bp_opt2,
        "pump_power_opt2": pump_power_opt2,
        "flowrate_opt2": flowrate_opt2,
        "coal_save2": coal_save2,
        "dpower2": dpower2,

    }

    true_res = {
        "power1": power1,
        "temp1": temp1,
        "power2": power2,
        "temp2": temp2,
    }

    cold_result.pass_result(opt_result=opt_res, true_state=true_res)
    cold_result.平抑波动(unit=[1, 2])
    output_res = cold_result.opt_result

    if not 计时器1号机.get("initial_state"):
        time_now = datetime.datetime.now()
        time_last_state1 = 计时器1号机.get("last_state_time1")
        time_last_state2 = 计时器1号机.get("last_state_time2")
        time_last_state3 = 计时器1号机.get("last_state_time3")
        time_last_state4 = 计时器1号机.get("last_state_time4")
        time_delta1 = time_now - time_last_state1
        time_delta2 = time_now - time_last_state2
        time_delta3 = time_now - time_last_state3
        time_delta4 = time_now - time_last_state4
        hour11, minute11 = time_delta_2_hour_minute(time_delta1)
        hour21, minute21 = time_delta_2_hour_minute(time_delta2)
        hour31, minute31 = time_delta_2_hour_minute(time_delta3)
        hour41, minute41 = time_delta_2_hour_minute(time_delta4)
        logger.debug(f"{hour11}:{minute11}   {hour21}:{minute21}   {hour31}:{minute31}    {hour41}:{minute41}")
    else:
        hour11, minute11 = 0, 0
        hour21, minute21 = 0, 0
        hour31, minute31 = 0, 0
        hour41, minute41 = 0, 0

    if not 计时器2号机.get("initial_state"):
        time_now = datetime.datetime.now()
        time_last_state1 = 计时器2号机.get("last_state_time1")
        time_last_state2 = 计时器2号机.get("last_state_time2")
        time_last_state3 = 计时器2号机.get("last_state_time3")
        time_last_state4 = 计时器2号机.get("last_state_time4")
        time_delta1 = time_now - time_last_state1
        time_delta2 = time_now - time_last_state2
        time_delta3 = time_now - time_last_state3
        time_delta4 = time_now - time_last_state4
        hour12, minute12 = time_delta_2_hour_minute(time_delta1)
        hour22, minute22 = time_delta_2_hour_minute(time_delta2)
        hour32, minute32 = time_delta_2_hour_minute(time_delta3)
        hour42, minute42 = time_delta_2_hour_minute(time_delta4)
    else:
        hour12, minute12 = 0, 0
        hour22, minute22 = 0, 0
        hour32, minute32 = 0, 0
        hour42, minute42 = 0, 0
    hp1, lp1 = get_pump_type_int(type1_)
    hp2, lp2 = get_pump_type_int(type2_)
    hp1_opt, lp1_opt = get_pump_type_int(type_opt1)
    hp2_opt, lp2_opt = get_pump_type_int(type_opt2)
    res = {
        "建议循泵运行方式11": output_res["pump11_unit1"],
        "建议循泵运行方式12": output_res["pump12_unit1"],
        "建议循泵运行方式13": output_res["pump13_unit1"],
        "建议循泵运行方式14": output_res["pump14_unit1"],
        "建议循泵运行方式21": output_res["pump21_unit2"],
        "建议循泵运行方式22": output_res["pump22_unit2"],
        "建议循泵运行方式23": output_res["pump23_unit2"],
        "建议循泵运行方式24": output_res["pump24_unit2"],
        "最优运行方式下背压1": output_res["bp_opt1"],
        "最优运行方式下背压2": output_res["bp_opt2"],
        "优化后机组节煤量1": output_res["coal_save1"],
        "优化后机组节煤量2": output_res["coal_save2"],
        "净电功率增益1": output_res["dpower1"],
        "净电功率增益2": output_res["dpower2"],
        "循环水流量1": output_res["flowrate_opt1"],
        "循环水流量2": output_res["flowrate_opt2"],
        "循泵总功率1": output_res["pump_power_opt1"],
        "循泵总功率2": output_res["pump_power_opt2"],
        "循泵切换小时数11": hour11,
        "循泵切换分钟数11": minute11,
        "循泵切换小时数21": hour21,
        "循泵切换分钟数21": minute21,
        "循泵切换小时数31": hour31,
        "循泵切换分钟数31": minute31,
        "循泵切换小时数41": hour41,
        "循泵切换分钟数41": minute41,
        "循泵切换小时数12": hour12,
        "循泵切换分钟数12": minute12,
        "循泵切换小时数22": hour22,
        "循泵切换分钟数22": minute22,
        "循泵切换小时数32": hour32,
        "循泵切换分钟数32": minute32,
        "循泵切换小时数42": hour42,
        "循泵切换分钟数42": minute42,

        "高速泵运行台数实时值1": hp1,
        "低速泵运行台数实时值1": lp1,
        "高速泵运行台数推荐值1": hp1_opt,
        "低速泵运行台数推荐值1": lp1_opt,
        "高速泵运行台数实时值2": hp2,
        "低速泵运行台数实时值2": lp2,
        "高速泵运行台数推荐值2": hp2_opt,
        "低速泵运行台数推荐值2": lp2_opt,
        "循泵总功率实际值1": pump_power1,
        "循泵总功率实际值2": pump_power2,

        "阻塞背压1": 0.0052 * power1 - 0.0283,
        "阻塞背压2": 0.0052 * power2 - 0.0283,
    }
    tags_values = {TagsWrite.get(k): res.get(k) for k in res.keys()}
    db.write_snapshot(tags=tags_values.keys(), values=tags_values.values())
    logger.debug("写入SIS成功")


if __name__ == "__main__":
    execute_function_by_interval(optimize, minute=0, second=30, daemon=True)



import math
from abc import abstractmethod

from yangke.common.config import logger

背景图层id = -10
管线图层id = -5
组件图层id = 0  # 图层id越小的图元，越先绘制


class DefaultUnit:  # 默认单位类
    def __init__(self):
        self.P = "MPa"
        self.T = "℃"
        self.MassFlow = "t/h"
        self.Power = "kW"
        self.Density = "kg/m^3"
        self.Enthalpy = "kJ/kg"
        self.VolumeFlow = "m^3/h"
        self.SpecificVolume = "m^3/kg"
        self.Length = "m"
        self.Area = "m^2"
        self.AngleUnit = "°"
        self.Velocity = "m/s"

    def set(self, units: dict):
        """
        根据字典信息设置默认的单位

        :param units:
        :return:
        """
        for symbol, unit_str in units.items():
            if symbol == "P":
                self.P = Pressure(unit_str)
            elif symbol == "T":
                self.T = Temperature(unit_str)
            elif symbol == "MassFlow":
                self.MassFlow = MassFlow(unit_str)
            elif symbol == "Power":
                self.Power = Power(unit_str)
            elif symbol == "Density":
                self.Density = Density(unit_str)
            elif symbol == "Enthalpy":
                self.Enthalpy = Enthalpy(unit_str)
            elif symbol == "VolumeFlow":
                self.VolumeFlow = VolumeFlow(unit_str)
            elif symbol == "SpecificVolume":
                self.SpecificVolume = SpecificVolume(unit_str)
            elif symbol == "Length":
                self.Length = Length(unit_str)
            elif symbol == "Area":
                self.Area = Area(unit_str)


class Unit:  # 单位的父类
    def __init__(self):
        self.allowed_values = []
        self.sf = []  # 不同单位对应的缩放因子，如果可以通过缩放转换单位，则定义该参数即可
        self.unit = None

    def get_values_list(self):
        return self.allowed_values

    def get_selected_value(self):
        return self.unit

    def __str__(self):
        return self.unit

    def get_scale_factor(self, unit_str):
        """
        当前单位转换为指定单位时需要乘以的系数,即，如果当前单位为Pressure("MPa")，unit_str="kPa"，则scale_factor=1000

        :param unit_str:
        :return:
        """
        zip_factors = dict(zip(self.allowed_values, self.sf))
        sf = zip_factors.get(self.unit) / zip_factors.get(unit_str)
        return sf

    def transfer_to_unit(self, value: float, unit: str):
        """
        如果单位可以通过简单的乘以系数完成转换，则定义self.sf后调用该方法即可，否则，应自己实现单位的转换方法。
        简单的乘以系数的示例如下方的Pressure类。
        自定义单位转换方法的示例如下方的Temperature类。

        :param value:
        :param unit:
        :return:
        """
        try:
            value = float(value)
        except:
            logger.debug(f"{value=}无法转换单位为{unit}")
        destination_value = self.get_scale_factor(unit) * value
        return destination_value


class Pressure(Unit):
    def __init__(self, unit=None):
        super(Pressure, self).__init__()
        self.allowed_values = ["MPa", "kPa", "Pa", "bar", "hPa"]
        self.sf = [1000000, 1000, 1, 101325, 100]
        if unit is None:
            self.unit = default_unit.P
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"压力单位不允许为{unit}")
            self.unit = "MPa"


class Velocity(Unit):
    def __init__(self, unit=None):
        super(Velocity, self).__init__()
        self.allowed_values = ["m/s", "km/h"]
        self.sf = [1, 1 / 3.6]
        if unit is None:
            self.unit = default_unit.Velocity
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"速度单位不允许为{unit}")
            self.unit = "MPa"


class Temperature(Unit):

    def __init__(self, unit=None):
        super(Temperature, self).__init__()
        self.allowed_values = ["℃", "K"]
        if unit is None:
            self.unit = default_unit.T
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"温度单位不允许为{unit}")
            self.unit = "℃"

    def get_scale_factor(self, unit_str):
        logger.warning(f"温度单位不支持缩放系数转换单位")

    def transfer_to_unit(self, value: float, unit: str):
        try:
            value = float(value)
        except:
            logger.debug(f"{value=}无法转换单位为{unit}")
        if self.unit == unit:
            res = value
        elif self.unit == "℃" and unit == "K":
            res = value + 273.15
        elif self.unit == "K" and unit == "℃":
            res = value - 273.15
        return res


class Enthalpy(Unit):

    def __init__(self, unit="kJ/kg"):
        super(Enthalpy, self).__init__()
        self.allowed_values = ["kJ/kg", "J/g"]
        self.sf = [1, 1]
        if unit is None:
            self.unit = default_unit.Enthalpy
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"焓单位不允许为{unit}")
            self.unit = "kJ/kg"


class MassFlow(Unit):

    def __init__(self, unit=None):
        super(MassFlow, self).__init__()
        self.allowed_values = ["t/h", "t/s", "kg/s"]
        self.sf = [1 / 3.6, 1000, 1]
        if unit is None:
            self.unit = default_unit.MassFlow
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"质量流量单位不允许为{unit}")
            self.unit = "t/h"


class VolumeFlow(Unit):
    def __init__(self, unit=None):
        super(VolumeFlow, self).__init__()
        self.allowed_values = ["m^3/h", "m^3/s"]
        self.sf = [1 / 3600, 1]
        if unit is None:
            self.unit = default_unit.VolumeFlow
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"体积流量单位不允许为{unit}")
            self.unit = "m^3/h"


class SpecificVolume(Unit):
    def __init__(self, unit=None):
        super(SpecificVolume, self).__init__()
        self.allowed_values = ["m^3/kg", ]
        self.sf = [1]
        if unit is None:
            self.unit = default_unit.SpecificVolume
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"比容单位不允许为{unit}")
            self.unit = "m^3/kg"


class Density(Unit):

    def __init__(self, unit=None):
        super(Density, self).__init__()
        self.allowed_values = ["kg/m^3", ]
        self.sf = [1]
        if unit is None:
            self.unit = default_unit.Density
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"密度单位不允许为{unit}")
            self.unit = "kg/m^3"


class Power(Unit):

    def __init__(self, unit=None):
        super(Power, self).__init__()
        self.allowed_values = ["MW", "kW", "W"]
        self.sf = [1000000, 1000, 1]
        if unit is None:
            self.unit = default_unit.Power
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"功率单位不允许为{unit}")
            self.unit = "MW"


class Length(Unit):
    def __init__(self, unit=None):
        super(Length, self).__init__()
        self.allowed_values = ["m", "mm", "km"]
        self.sf = [1, 0.001, 1000]
        if unit is None:
            self.unit = default_unit.Length
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"长度单位不允许为{unit}")
            self.unit = "m"


class Area(Unit):
    def __init__(self, unit=None):
        super(Area, self).__init__()
        self.allowed_values = ["m^2", "mm^2", "km^2"]
        self.sf = [1, 0.000001, 1000000]
        if unit is None:
            self.unit = default_unit.Area
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"长度单位不允许为{unit}")
            self.unit = "m^2"


class AngleUnit(Unit):
    def __init__(self, unit=None):
        """
        角度的单位，包括°和rad
        :param unit:
        """
        super(AngleUnit, self).__init__()
        self.allowed_values = ["°", "rad"]
        self.sf = [math.pi / 180, 1]  # 1° = 180/pi rad <== 180° = pi rad
        if unit is None:
            self.unit = default_unit.AngleUnit
        elif unit in self.allowed_values:
            self.unit = unit
        else:
            logger.error(f"角度单位不允许为{unit}")
            self.unit = None


class One(Unit):
    def __int__(self, unit=None):
        """
        无单位的单位，即单位1

        :param unit:
        :return:
        """
        super(One, self).__init__()
        self.allowed_values = ["%", "1", "‰"]  # 不能为None，没有单位，单位就是1，而None表示参数不是数值型，如类型等，这种才是真正的没有单位
        self.sf = [0.01, 1, 0.001, 1]
        if unit in self.allowed_values:
            self.unit = unit
        else:
            self.unit = "1"


class Other(Unit):
    def __init__(self, unit):
        super(Other, self).__init__()
        self.unit = unit


default_unit = DefaultUnit()  # 放到最后初始化，因为该参数的初始化必须等所有的类都加载后才能进行



from yangke.core import runCMD, str_in_list


def start_neo4j():
    if neo4j_available():
        return True
    run_result, run_err = runCMD("neo4j start", output_type="REALTIME_RETURN")
    if neo4j_available():
        return True
    else:
        return False


def stop_neo4j():
    if not neo4j_available():
        return True
    run_result, run_err = runCMD("neo4j stop", output_type="REALTIME_RETURN")
    if neo4j_available():
        return False
    else:
        return True


def neo4j_available():
    run_result, run_err = runCMD("neo4j status", output_type="REALTIME_RETURN")
    if str_in_list("Neo4j is running", run_result):
        return True
    elif str_in_list("Neo4j is not running", run_result):
        return False



import os

from yangke.common.QtImporter import Qt, QModelIndex, QSize, QRect, QFont, QPen, QColor, QFileDialog, QComboBox, \
    QStyleOptionViewItem, QStyle, QPainter, QPointF, QRectF, QPainterPath, QStyledItemDelegate


class StockItemDelegate(QStyledItemDelegate):
    def __init__(self):
        """
        【所有组件】面板中所有组件的显示方式
        """
        super().__init__()
        self.width = 220
        self.height = 25

    def paint(self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex):
        if index.isValid():
            painter.save()

            color_primary = QColor(os.getenv('QTMATERIAL_PRIMARYCOLOR'))
            color_light_primary = QColor(os.getenv('QTMATERIAL_PRIMARYLIGHTCOLOR'))
            color_secondary = QColor(os.getenv('QTMATERIAL_SECONDARYCOLOR'))
            color_light_secondary = QColor(os.getenv('QTMATERIAL_SECONDARYLIGHTCOLOR'))
            color_dark_secondary = QColor(os.getenv('QTMATERIAL_SECONDARYDARKCOLOR'))
            color_primary_text = QColor(os.getenv('QTMATERIAL_PRIMARYTEXTCOLOR'))
            color_secondary_text = QColor(os.getenv('QTMATERIAL_SECONDARYTEXTCOLOR'))

            user_data = index.data(Qt.UserRole)
            symbol = user_data.get("name")  # 组件在Ebsilon中的id
            name = user_data.get("名称")
            rect = QRectF()
            rect.setX(option.rect.x())
            rect.setY(option.rect.y())
            rect.setWidth(self.width - 1)
            rect.setHeight(self.height - 1)

            # 绘制圆角矩形
            radius = 7
            path: QPainterPath = QPainterPath()
            path.moveTo(rect.topRight() - QPointF(radius, 0))
            path.lineTo(rect.topLeft() + QPointF(radius, 0))
            path.quadTo(rect.topLeft(), rect.topLeft() + QPointF(0, radius))
            path.lineTo(rect.bottomLeft() + QPointF(0, -radius))
            path.quadTo(rect.bottomLeft(), rect.bottomLeft() + QPointF(radius, 0))
            path.lineTo(rect.bottomRight() - QPointF(radius, 0))
            path.quadTo(rect.bottomRight(), rect.bottomRight() + QPointF(0, -radius))
            path.lineTo(rect.topRight() + QPointF(0, radius))  # 8
            path.quadTo(rect.topRight(), rect.topRight() + QPointF(-radius, -0))

            # 绘制数据位置
            symbol_rect = QRect(int(rect.left() + 5), int(rect.top() + 5), int(rect.width() - 30), 20)  # 中文组件名所在的区域
            name_rect = QRect(int(rect.right() - 100), int(rect.top() + 5), int(rect.width() - 100), 20)  # EBS_ID

            if option.state & QStyle.State_Selected:
                painter.setPen(QPen(color_secondary))
                painter.setBrush(color_primary)
                painter.drawPath(path)
            elif option.state & QStyle.State_MouseOver:
                painter.setPen(color_primary)
                painter.setBrush(color_light_secondary)
                painter.drawPath(path)
            else:
                painter.setPen(QPen(color_light_primary))
                painter.setBrush(Qt.NoBrush)
                painter.drawPath(path)

            # 绘制组件名
            # painter.drawEllipse(circle)
            painter.setPen(QPen(color_secondary_text))
            painter.setFont(QFont("Times", 10, QFont.Bold))
            painter.drawText(symbol_rect, Qt.AlignLeft, symbol)  # 绘制中文组件名

            # 绘制右侧的EBS_ID
            painter.setPen(QPen(color_secondary_text))
            painter.setFont(QFont("Times", 10))
            painter.drawText(name_rect, Qt.AlignLeft, name)

            painter.restore()

    def sizeHint(self, option: QStyleOptionViewItem, index: QModelIndex):
        return QSize(self.width, self.height)
